@@ -1,7 +1,7 @@
 from typing import NamedTuple
 
 from src.cameras.multicam_manager import CVCameraManager
-from src.core_processor.board_detection.board_detection import BoardDetection
+from src.core_processor.camera_calibration.charuco_board_detection.charuco_board_detector import CharucoBoardDetector
 
 
 class ImageResponse(NamedTuple):
@@ -14,7 +14,7 @@ class BoardDetectService:
         self._cam_manager = cam_manager
 
     async def run(self):
-        return BoardDetection(self._cam_manager).process()
+        return CharucoBoardDetector(self._cam_manager).process()
 
     async def run_detection_on_cam_id(self, webcam_id: str, cb=None):
-        await BoardDetection(self._cam_manager).process_by_cam_id(webcam_id, cb)
+        await CharucoBoardDetector(self._cam_manager).process_by_cam_id(webcam_id, cb)
